import { transformSchemaAST } from '@graphql-codegen/schema-ast';
import { visit } from 'graphql';
import { topologicalSortAST } from './graphql';
import { MyZodSchemaVisitor } from './myzod/index';
import { YupSchemaVisitor } from './yup/index';
import { ZodSchemaVisitor } from './zod/index';
export const plugin = (schema, _documents, config) => {
    const { schema: _schema, ast } = _transformSchemaAST(schema, config);
    const { buildImports, initialEmit, ...visitor } = schemaVisitor(_schema, config);
    const result = visit(ast, visitor);
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    const generated = result.definitions.filter(def => typeof def === 'string');
    return {
        prepend: buildImports(),
        content: [initialEmit(), ...generated].join('\n'),
    };
};
const schemaVisitor = (schema, config) => {
    if (config?.schema === 'zod') {
        return ZodSchemaVisitor(schema, config);
    }
    else if (config?.schema === 'myzod') {
        return MyZodSchemaVisitor(schema, config);
    }
    return YupSchemaVisitor(schema, config);
};
const _transformSchemaAST = (schema, config) => {
    const { schema: _schema, ast } = transformSchemaAST(schema, config);
    // This affects the performance of code generation, so it is
    // enabled only when this option is selected.
    if (config.validationSchemaExportType === 'const') {
        return {
            schema: _schema,
            ast: topologicalSortAST(_schema, ast),
        };
    }
    return {
        schema: _schema,
        ast,
    };
};
